@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """
 raven.contrib.django.client
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -8,6 +9,7 @@ raven.contrib.django.client
 
 from __future__ import absolute_import
 
+import time
 import logging
 
 from django.conf import settings
@@ -26,16 +28,119 @@ from raven.base import Client
 from raven.contrib.django.utils import get_data_from_template, get_host
 from raven.contrib.django.middleware import SentryLogMiddleware
 from raven.utils.wsgi import get_headers, get_environ
+from raven.utils import once
+from raven import breadcrumbs
+from raven._compat import string_types, binary_type
 
 __all__ = ('DjangoClient',)
 
 
+class _FormatConverter(object):
+
+    def __init__(self, param_mapping):
+        self.param_mapping = param_mapping
+        self.params = []
+
+    def __getitem__(self, val):
+        self.params.append(self.param_mapping.get(val))
+        return '%s'
+
+
+def format_sql(sql, params):
+    rv = []
+
+    if isinstance(params, dict):
+        conv = _FormatConverter(params)
+        sql = sql % conv
+        params = conv.params
+
+    for param in params or ():
+        if param is None:
+            rv.append('NULL')
+        elif isinstance(param, string_types):
+            if isinstance(param, binary_type):
+                param = param.decode('utf-8', 'replace')
+            if len(param) > 256:
+                param = param[:256] + u'…'
+            rv.append("'%s'" % param.replace("'", "''"))
+        else:
+            rv.append(repr(param))
+
+    return sql, rv
+
+
+@once
+def install_sql_hook():
+    """If installed this causes Django's queries to be captured."""
+    try:
+        from django.db.backends.utils import CursorWrapper
+    except ImportError:
+        from django.db.backends.util import CursorWrapper
+
+    try:
+        real_execute = CursorWrapper.execute
+        real_executemany = CursorWrapper.executemany
+    except AttributeError:
+        # XXX(mitsuhiko): On some very old django versions (<1.6) this
+        # trickery would have to look different but I can't be bothered.
+        return
+
+    def record_sql(vendor, alias, start, duration, sql, params):
+        def processor(data):
+            real_sql, real_params = format_sql(sql, params)
+            if real_params:
+                real_sql = real_sql % tuple(real_params)
+            # maybe category to 'django.%s.%s' % (vendor, alias or
+            #   'default') ?
+            data.update({
+                'message': real_sql,
+                'duration': duration,
+                'category': 'query',
+            })
+        breadcrumbs.record_breadcrumb('default', processor=processor)
+
+    def record_many_sql(vendor, alias, start, sql, param_list):
+        duration = time.time() - start
+        for params in param_list:
+            record_sql(vendor, alias, start, duration, sql, params)
+
+    def execute(self, sql, params=None):
+        start = time.time()
+        try:
+            return real_execute(self, sql, params)
+        finally:
+            record_sql(self.db.vendor, getattr(self.db, 'alias', None),
+                       start, time.time() - start, sql, params)
+
+    def executemany(self, sql, param_list):
+        start = time.time()
+        try:
+            return real_executemany(self, sql, param_list)
+        finally:
+            record_many_sql(self.db.vendor, getattr(self.db, 'alias', None),
+                            start, sql, param_list)
+
+    CursorWrapper.execute = execute
+    CursorWrapper.executemany = executemany
+    breadcrumbs.ignore_logger('django.db.backends')
+
+
 class DjangoClient(Client):
     logger = logging.getLogger('sentry.errors.client.django')
 
+    def __init__(self, *args, **kwargs):
+        install_sql_hook = kwargs.pop('install_sql_hook', True)
+        Client.__init__(self, *args, **kwargs)
+        if install_sql_hook:
+            self.install_sql_hook()
+
+    def install_sql_hook(self):
+        install_sql_hook()
+
     def get_user_info(self, user):
-        if not user.is_authenticated():
-            return {}
+        if hasattr(user, 'is_authenticated') and \
+           not user.is_authenticated():
+            return None
 
         user_info = {
             'id': user.pk,
@@ -52,19 +157,13 @@ class DjangoClient(Client):
         return user_info
 
     def get_data_from_request(self, request):
-        try:
-            from django.contrib.auth.models import AbstractBaseUser as BaseUser
-        except ImportError:
-            from django.contrib.auth.models import User as BaseUser  # NOQA
-        except RuntimeError:
-            # If the contenttype / user applications are not installed trying to
-            # import the user models will fail for django >= 1.9.
-            BaseUser = None
-
         result = {}
 
-        if BaseUser and hasattr(request, 'user') and isinstance(request.user, BaseUser):
-            result['user'] = self.get_user_info(request.user)
+        user = getattr(request, 'user', None)
+        if user is not None:
+            user_info = self.get_user_info(user)
+            if user_info:
+                result['user'] = user_info
 
         try:
             uri = request.build_absolute_uri()
